@@ -23,22 +23,18 @@
  * SECTION:element-textrender
  * @see_also: #GstTextOverlay
  *
- * <refsect2>
- * <para>
  * This plugin renders text received on the text sink pad to a video
  * buffer (retaining the alpha channel), so it can later be overlayed
  * on top of video streams using other elements.
- * </para>
- * <para>
+ *
  * The text can contain newline characters. (FIXME: What about text 
  * wrapping? It does not make sense in this context)
- * </para>
- * <para>
- * Example pipeline:
- * <programlisting>
+ *
+ * <refsect2>
+ * <title>Example launch lines</title>
+ * |[
  * gst-launch -v filesrc location=subtitles.srt ! subparse ! textrender ! xvimagesink
- * </programlisting>
- * </para>
+ * ]|
  * </refsect2>
  */
 
@@ -54,13 +50,12 @@
 GST_DEBUG_CATEGORY_EXTERN (pango_debug);
 #define GST_CAT_DEFAULT pango_debug
 
-static GstElementDetails text_render_details = {
-  "Text Render",
-  "Filter/Editor/Video",
-  "Renders a text string to an image bitmap",
-  "David Schleef <ds@schleef.org>, "
-      "Ronald S. Bultje <rbultje@ronald.bitfreak.net>"
-};
+static const GstElementDetails text_render_details =
+GST_ELEMENT_DETAILS ("Text renderer",
+    "Filter/Editor/Video",
+    "Renders a text string to an image bitmap",
+    "David Schleef <ds@schleef.org>, "
+    "Ronald S. Bultje <rbultje@ronald.bitfreak.net>");
 
 enum
 {
@@ -122,7 +117,7 @@ gst_text_render_class_init (GstTextRenderClass * klass)
           "to be used for rendering. "
           "See documentation of "
           "pango_font_description_from_string"
-          " for syntax.", "", G_PARAM_WRITABLE));
+          " for syntax.", "", G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 }
 
 
@@ -287,7 +282,7 @@ gst_text_render_chain (GstPad * pad, GstBuffer * inbuf)
   if (ret != GST_FLOW_OK)
     goto done;
 
-  gst_buffer_stamp (outbuf, inbuf);
+  gst_buffer_copy_metadata (outbuf, inbuf, GST_BUFFER_COPY_TIMESTAMPS);
   data = GST_BUFFER_DATA (outbuf);
 
   for (n = 0; n < render->width * render->height; n++) {
