@@ -22,30 +22,23 @@
  * SECTION:element-timeoverlay
  * @see_also: #GstTextOverlay, #GstClockOverlay
  *
- * <refsect2>
- * <para>
  * This element overlays the buffer time stamps of a video stream on
  * top of itself. You can position the text and configure the font details
  * using the properties of the #GstTextOverlay class. By default, the
  * time stamp is displayed in the top left corner of the picture, with some
  * padding to the left and to the top.
- * </para>
- * <para>
- * Here is a simple pipeline that displays the time stamps in the top left
- * corner of the video picture:
- * <programlisting>
+ *
+ * <refsect2>
+ * |[
  * gst-launch -v videotestsrc ! timeoverlay ! xvimagesink
- * </programlisting>
- * </para>
- * <para>
- * Here is another pipeline that displays the time stamps with some leading
+ * ]| Display the time stamps in the top left
+ * corner of the video picture.
+ * |[
+ * gst-launch -v videotestsrc ! timeoverlay halign=right valign=bottom text="Stream time:" shaded-background=true ! xvimagesink
+ * ]| Another pipeline that displays the time stamps with some leading
  * text in the bottom right corner of the video picture, with the background
  * of the text being shaded in order to make it more legible on top of a
- * bright video background:
- * <programlisting>
- * gst-launch -v videotestsrc ! timeoverlay halign=right valign=bottom text="Stream time:" shaded-background=true ! xvimagesink
- * </programlisting>
- * </para>
+ * bright video background.
  * </refsect2>
  */
 
@@ -57,7 +50,7 @@
 
 #include <gsttimeoverlay.h>
 
-static GstElementDetails time_overlay_details =
+static const GstElementDetails time_overlay_details =
 GST_ELEMENT_DETAILS ("Time overlay",
     "Filter/Editor/Video",
     "Overlays buffer time stamps on a video stream",
@@ -146,7 +139,7 @@ gst_time_overlay_init (GstTimeOverlay * overlay, GstTimeOverlayClass * klass)
   pango_context_set_base_dir (context, PANGO_DIRECTION_LTR);
 
   font_description = pango_font_description_new ();
-  pango_font_description_set_family (font_description, g_strdup ("Monospace"));
+  pango_font_description_set_family_static (font_description, "Monospace");
   pango_font_description_set_style (font_description, PANGO_STYLE_NORMAL);
   pango_font_description_set_variant (font_description, PANGO_VARIANT_NORMAL);
   pango_font_description_set_weight (font_description, PANGO_WEIGHT_NORMAL);
