@@ -21,33 +21,30 @@
  * SECTION:element-theoraenc
  * @see_also: theoradec, oggmux
  *
- * <refsect2>
- * <para>
  * This element encodes raw video into a Theora stream.
  * <ulink url="http://www.theora.org/">Theora</ulink> is a royalty-free
  * video codec maintained by the <ulink url="http://www.xiph.org/">Xiph.org
  * Foundation</ulink>, based on the VP3 codec.
- * </para>
- * <para>
+ *
  * The theora codec internally only supports encoding of images that are a
  * multiple of 16 pixels in both X and Y direction. It is however perfectly
  * possible to encode images with other dimensions because an arbitrary
  * rectangular cropping region can be set up. This element will automatically
  * set up a correct cropping region if the dimensions are not multiples of 16
- * pixels. The "border" and "center" properties control how this cropping
- * region will be set up.
- * </para>
- * <para>
- * To control the quality of the encoding, the "bitrate" and "quality"
- * properties can be used. These two properties are mutualy exclusive. Setting
- * the bitrate property will produce a constant bitrate (CBR) stream while
- * setting the quality property will produce a variable bitrate (VBR) stream.
- * </para>
+ * pixels. The #GstTheoraEnc::border and #GstTheoraEnc::center properties
+ * control how this cropping region will be set up.
+ *
+ * To control the quality of the encoding, the #GstTheoraEnc::bitrate and
+ * #GstTheoraEnc::quality properties can be used. These two properties are
+ * mutualy exclusive. Setting the bitrate property will produce a constant
+ * bitrate (CBR) stream while setting the quality property will produce a
+ * variable bitrate (VBR) stream.
+ *
+ * <refsect2>
  * <title>Example pipeline</title>
- * <programlisting>
+ * |[
  * gst-launch -v videotestsrc num-buffers=1000 ! theoraenc ! oggmux ! filesink location=videotestsrc.ogg
- * </programlisting>
- * This example pipeline will encode a test video source to theora muxed in an
+ * ]| This example pipeline will encode a test video source to theora muxed in an
  * ogg container. Refer to the theoradec documentation to decode the create
  * stream.
  * </refsect2>
@@ -220,50 +217,54 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
   g_object_class_install_property (gobject_class, ARG_CENTER,
       g_param_spec_boolean ("center", "Center",
           "Center image when sizes not multiple of 16", THEORA_DEF_CENTER,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_BORDER,
       g_param_spec_enum ("border", "Border",
           "Border color to add when sizes not multiple of 16",
           GST_TYPE_BORDER_MODE, THEORA_DEF_BORDER,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /* general encoding stream options */
   g_object_class_install_property (gobject_class, ARG_BITRATE,
       g_param_spec_int ("bitrate", "Bitrate", "Compressed video bitrate (kbps)",
-          0, 2000, THEORA_DEF_BITRATE, (GParamFlags) G_PARAM_READWRITE));
+          0, (1 << 24) - 1, THEORA_DEF_BITRATE,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_QUALITY,
-      g_param_spec_int ("quality", "Quality", "Video quality",
-          0, 63, THEORA_DEF_QUALITY, (GParamFlags) G_PARAM_READWRITE));
+      g_param_spec_int ("quality", "Quality", "Video quality", 0, 63,
+          THEORA_DEF_QUALITY,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_QUICK,
       g_param_spec_boolean ("quick", "Quick", "Quick encoding",
-          THEORA_DEF_QUICK, (GParamFlags) G_PARAM_READWRITE));
+          THEORA_DEF_QUICK,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_KEYFRAME_AUTO,
       g_param_spec_boolean ("keyframe-auto", "Keyframe Auto",
           "Automatic keyframe detection", THEORA_DEF_KEYFRAME_AUTO,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_KEYFRAME_FREQ,
       g_param_spec_int ("keyframe-freq", "Keyframe frequency",
           "Keyframe frequency", 1, 32768, THEORA_DEF_KEYFRAME_FREQ,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_KEYFRAME_FREQ_FORCE,
       g_param_spec_int ("keyframe-force", "Keyframe force",
           "Force keyframe every N frames", 1, 32768,
-          THEORA_DEF_KEYFRAME_FREQ_FORCE, (GParamFlags) G_PARAM_READWRITE));
+          THEORA_DEF_KEYFRAME_FREQ_FORCE,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_KEYFRAME_THRESHOLD,
       g_param_spec_int ("keyframe-threshold", "Keyframe threshold",
           "Keyframe threshold", 0, 32768, THEORA_DEF_KEYFRAME_THRESHOLD,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_KEYFRAME_MINDISTANCE,
       g_param_spec_int ("keyframe-mindistance", "Keyframe mindistance",
           "Keyframe mindistance", 1, 32768, THEORA_DEF_KEYFRAME_MINDISTANCE,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_NOISE_SENSITIVITY,
       g_param_spec_int ("noise-sensitivity", "Noise sensitivity",
           "Noise sensitivity", 0, 32768, THEORA_DEF_NOISE_SENSITIVITY,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_SHARPNESS,
-      g_param_spec_int ("sharpness", "Sharpness",
-          "Sharpness", 0, 2, THEORA_DEF_SHARPNESS,
-          (GParamFlags) G_PARAM_READWRITE));
+      g_param_spec_int ("sharpness", "Sharpness", "Sharpness", 0, 2,
+          THEORA_DEF_SHARPNESS,
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = theora_enc_change_state;
   GST_DEBUG_CATEGORY_INIT (theoraenc_debug, "theoraenc", 0, "Theora encoder");
