@@ -19,23 +19,18 @@
 
 /**
  * SECTION:element-vorbisdec
- * @short_description: a decoder that decodes Vorbis to raw audio
  * @see_also: vorbisenc, oggdemux
  *
- * <refsect2>
- * <para>
  * This element decodes a Vorbis stream to raw float audio.
  * <ulink url="http://www.vorbis.com/">Vorbis</ulink> is a royalty-free
  * audio codec maintained by the <ulink url="http://www.xiph.org/">Xiph.org
  * Foundation</ulink>.
- * </para>
+ *
+ * <refsect2>
  * <title>Example pipelines</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! alsasink
- * </programlisting>
- * Decode an Ogg/Vorbis. To create an Ogg/Vorbis file refer to the documentation of vorbisenc.
- * </para>
+ * ]| Decode an Ogg/Vorbis. To create an Ogg/Vorbis file refer to the documentation of vorbisenc.
  * </refsect2>
  *
  * Last reviewed on 2006-03-01 (0.10.4)
@@ -618,8 +613,49 @@ vorbis_handle_identification_packet (GstVorbisDec * vd)
       pos = pos6;
       break;
     }
-    default:
-      goto channel_count_error;
+      /* FIXME: for >6 channels the layout is not defined by the Vorbis
+       * spec. These are the gstreamer "defaults" for 7/8 channels and
+       * NONE layouts for more channels
+       */
+    case 7:{
+      static const GstAudioChannelPosition pos7[] = {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_LFE,
+        GST_AUDIO_CHANNEL_POSITION_REAR_CENTER,
+      };
+      pos = pos7;
+    }
+    case 8:{
+      static const GstAudioChannelPosition pos8[] = {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_LFE,
+        GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT,
+      };
+
+      pos = pos8;
+    }
+    default:{
+      gint i;
+      GstAudioChannelPosition *posn =
+          g_new (GstAudioChannelPosition, vd->vi.channels);
+
+      GST_ELEMENT_WARNING (GST_ELEMENT (vd), STREAM, DECODE,
+          (NULL), ("Using NONE channel layout for more than 8 channels"));
+
+      for (i = 0; i < vd->vi.channels; i++)
+        posn[i] = GST_AUDIO_CHANNEL_POSITION_NONE;
+
+      pos = posn;
+    }
   }
 
   caps = gst_caps_new_simple ("audio/x-raw-float",
@@ -630,18 +666,15 @@ vorbis_handle_identification_packet (GstVorbisDec * vd)
   if (pos) {
     gst_audio_set_channel_positions (gst_caps_get_structure (caps, 0), pos);
   }
+
+  if (vd->vi.channels > 8) {
+    g_free ((GstAudioChannelPosition *) pos);
+  }
+
   gst_pad_set_caps (vd->srcpad, caps);
   gst_caps_unref (caps);
 
   return GST_FLOW_OK;
-
-  /* ERROR */
-channel_count_error:
-  {
-    GST_ELEMENT_ERROR (vd, STREAM, NOT_IMPLEMENTED, (NULL),
-        ("Unsupported channel count %d", vd->vi.channels));
-    return GST_FLOW_ERROR;
-  }
 }
 
 static GstFlowReturn
@@ -649,7 +682,7 @@ vorbis_handle_comment_packet (GstVorbisDec * vd, ogg_packet * packet)
 {
   guint bitrate = 0;
   gchar *encoder = NULL;
-  GstTagList *list;
+  GstTagList *list, *old_list;
   GstBuffer *buf;
 
   GST_DEBUG_OBJECT (vd, "parsing comment packet");
@@ -661,8 +694,11 @@ vorbis_handle_comment_packet (GstVorbisDec * vd, ogg_packet * packet)
       gst_tag_list_from_vorbiscomment_buffer (buf, (guint8 *) "\003vorbis", 7,
       &encoder);
 
+  old_list = vd->taglist;
   vd->taglist = gst_tag_list_merge (vd->taglist, list, GST_TAG_MERGE_REPLACE);
 
+  if (old_list)
+    gst_tag_list_free (old_list);
   gst_tag_list_free (list);
   gst_buffer_unref (buf);
 
@@ -806,7 +842,12 @@ static GstFlowReturn
 vorbis_dec_push_forward (GstVorbisDec * dec, GstBuffer * buf)
 {
   GstFlowReturn result;
-  gint64 outoffset = GST_BUFFER_OFFSET (buf);
+  gint64 outoffset, origoffset;
+
+  origoffset = GST_BUFFER_OFFSET (buf);
+
+again:
+  outoffset = origoffset;
 
   if (outoffset == -1) {
     dec->queued = g_list_append (dec->queued, buf);
@@ -814,7 +855,7 @@ vorbis_dec_push_forward (GstVorbisDec * dec, GstBuffer * buf)
     result = GST_FLOW_OK;
   } else {
     if (G_UNLIKELY (dec->queued)) {
-      gint64 size;
+      guint size;
       GstClockTime ts;
       GList *walk;
 
@@ -827,9 +868,20 @@ vorbis_dec_push_forward (GstVorbisDec * dec, GstBuffer * buf)
       for (walk = g_list_last (dec->queued); walk;
           walk = g_list_previous (walk)) {
         GstBuffer *buffer = GST_BUFFER (walk->data);
+        guint offset;
 
-        outoffset -=
-            GST_BUFFER_SIZE (buffer) / (sizeof (float) * dec->vi.channels);
+        offset = GST_BUFFER_SIZE (buffer) / (sizeof (float) * dec->vi.channels);
+
+        if (outoffset >= offset)
+          outoffset -= offset;
+        else {
+          /* we can't go below 0, this means this first offset was at the eos
+           * page and we need to clip to it instead */
+          GST_DEBUG_OBJECT (dec, "clipping %" G_GINT64_FORMAT,
+              offset - outoffset);
+          origoffset += (offset - outoffset);
+          goto again;
+        }
 
         GST_BUFFER_OFFSET (buffer) = outoffset;
         GST_BUFFER_TIMESTAMP (buffer) =
@@ -837,9 +889,9 @@ vorbis_dec_push_forward (GstVorbisDec * dec, GstBuffer * buf)
         GST_BUFFER_DURATION (buffer) = GST_CLOCK_DIFF (GST_BUFFER_TIMESTAMP
             (buffer), ts);
         ts = GST_BUFFER_TIMESTAMP (buffer);
-        GST_DEBUG_OBJECT (dec, "patch buffer %" G_GUINT64_FORMAT
-            ", offset %" G_GUINT64_FORMAT ", timestamp %" GST_TIME_FORMAT
-            ", duration %" GST_TIME_FORMAT, size, outoffset,
+        GST_DEBUG_OBJECT (dec, "patch buffer %u, offset %" G_GUINT64_FORMAT
+            ", timestamp %" GST_TIME_FORMAT ", duration %" GST_TIME_FORMAT,
+            size, outoffset,
             GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
             GST_TIME_ARGS (GST_BUFFER_DURATION (buffer)));
         size--;
@@ -897,7 +949,7 @@ vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet)
   guint sample_count;
   GstBuffer *out;
   GstFlowReturn result;
-  GstClockTime timestamp = -1, nextts;
+  GstClockTime timestamp = GST_CLOCK_TIME_NONE, nextts;
   gint size;
 
   if (!vd->initialized)
@@ -965,8 +1017,7 @@ vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet)
         GST_TIME_ARGS (GST_BUFFER_DURATION (out)),
         GST_TIME_ARGS (vd->cur_timestamp + GST_BUFFER_DURATION (out)));
     vd->cur_timestamp += GST_BUFFER_DURATION (out);
-    GST_BUFFER_OFFSET (out) = GST_CLOCK_TIME_TO_FRAMES (vd->cur_timestamp,
-        vd->vi.rate);
+    GST_BUFFER_OFFSET (out) = GST_CLOCK_TIME_TO_FRAMES (timestamp, vd->vi.rate);
     GST_BUFFER_OFFSET_END (out) = GST_BUFFER_OFFSET (out) + sample_count;
   } else {
     /* we have incoming granulepos */
@@ -1150,7 +1201,7 @@ empty_header:
  *    output queue:    
  *
  * Then we decode each buffer in the decode queue in order and put the output
- * buffer in the output queue. The first buffer (7) will not produce and output
+ * buffer in the output queue. The first buffer (7) will not produce any output
  * because it needs the previous buffer (6) which did not arrive yet. This
  * yields:
  *
@@ -1376,6 +1427,7 @@ vorbis_dec_change_state (GstElement * element, GstStateChange transition)
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       GST_DEBUG_OBJECT (vd, "PAUSED -> READY, clearing vorbis structures");
+      vd->initialized = FALSE;
       vorbis_block_clear (&vd->vb);
       vorbis_dsp_clear (&vd->vd);
       vorbis_comment_clear (&vd->vc);
