@@ -19,30 +19,22 @@
 
 /**
  * SECTION:element-vorbisenc
- * @short_description: an encoder that encodes audio to Vorbis
  * @see_also: vorbisdec, oggmux
  *
- * <refsect2>
- * <para>
  * This element encodes raw float audio into a Vorbis stream.
  * <ulink url="http://www.vorbis.com/">Vorbis</ulink> is a royalty-free
  * audio codec maintained by the <ulink url="http://www.xiph.org/">Xiph.org
  * Foundation</ulink>.
- * </para>
+ *
+ * <refsect2>
  * <title>Example pipelines</title>
- * <para>
- * Encode a test sine signal to Ogg/Vorbis.  Note that the resulting file
- * will be really small because a sine signal compresses very well.
- * </para>
- * <programlisting>
+ * |[
  * gst-launch -v audiotestsrc wave=sine num-buffers=100 ! audioconvert ! vorbisenc ! oggmux ! filesink location=sine.ogg
- * </programlisting>
- * <para>
- * Record from a sound card using ALSA and encode to Ogg/Vorbis.
- * </para>
- * <programlisting>
+ * ]| Encode a test sine signal to Ogg/Vorbis.  Note that the resulting file
+ * will be really small because a sine signal compresses very well.
+ * |[
  * gst-launch -v alsasrc ! audioconvert ! vorbisenc ! oggmux ! filesink location=alsasrc.ogg
- * </programlisting>
+ * ]| Record from a sound card using ALSA and encode to Ogg/Vorbis.
  * </refsect2>
  *
  * Last reviewed on 2006-03-01 (0.10.4)
@@ -151,7 +143,7 @@ vorbis_caps_factory (void)
 static GstCaps *
 raw_caps_factory (void)
 {
-  /* lowest, highest sample rates come from vorbis/lib/modes/setup_X.h: 
+  /* lowest, highest sample rates come from vorbis/lib/modes/setup_X.h:
    * 1-200000 Hz */
   return
       gst_caps_new_simple ("audio/x-raw-float",
@@ -197,28 +189,32 @@ gst_vorbis_enc_class_init (GstVorbisEncClass * klass)
       g_param_spec_int ("max-bitrate", "Maximum Bitrate",
           "Specify a maximum bitrate (in bps). Useful for streaming "
           "applications. (-1 == disabled)",
-          -1, HIGHEST_BITRATE, MAX_BITRATE_DEFAULT, G_PARAM_READWRITE));
+          -1, HIGHEST_BITRATE, MAX_BITRATE_DEFAULT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BITRATE,
       g_param_spec_int ("bitrate", "Target Bitrate",
           "Attempt to encode at a bitrate averaging this (in bps). "
           "This uses the bitrate management engine, and is not recommended for most users. "
-          "Quality is a better alternative. (-1 == disabled)",
-          -1, HIGHEST_BITRATE, BITRATE_DEFAULT, G_PARAM_READWRITE));
+          "Quality is a better alternative. (-1 == disabled)", -1,
+          HIGHEST_BITRATE, BITRATE_DEFAULT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MIN_BITRATE,
-      g_param_spec_int ("min_bitrate", "Minimum Bitrate",
+      g_param_spec_int ("min-bitrate", "Minimum Bitrate",
           "Specify a minimum bitrate (in bps). Useful for encoding for a "
-          "fixed-size channel. (-1 == disabled)",
-          -1, HIGHEST_BITRATE, MIN_BITRATE_DEFAULT, G_PARAM_READWRITE));
+          "fixed-size channel. (-1 == disabled)", -1, HIGHEST_BITRATE,
+          MIN_BITRATE_DEFAULT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_QUALITY,
       g_param_spec_float ("quality", "Quality",
-          "Specify quality instead of specifying a particular bitrate.",
-          -0.1, 1.0, QUALITY_DEFAULT, G_PARAM_READWRITE));
+          "Specify quality instead of specifying a particular bitrate.", -0.1,
+          1.0, QUALITY_DEFAULT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MANAGED,
       g_param_spec_boolean ("managed", "Managed",
-          "Enable bitrate management engine", FALSE, G_PARAM_READWRITE));
+          "Enable bitrate management engine", FALSE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_LAST_MESSAGE,
       g_param_spec_string ("last-message", "last-message",
-          "The last status message", NULL, G_PARAM_READABLE));
+          "The last status message", NULL,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_vorbis_enc_change_state);
@@ -237,7 +233,7 @@ gst_vorbis_enc_dispose (GObject * object)
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
-static const GstAudioChannelPosition vorbischannelpositions[][6] = {
+static const GstAudioChannelPosition vorbischannelpositions[][8] = {
   {                             /* Mono */
       GST_AUDIO_CHANNEL_POSITION_FRONT_MONO},
   {                             /* Stereo */
@@ -268,7 +264,27 @@ static const GstAudioChannelPosition vorbischannelpositions[][6] = {
         GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
         GST_AUDIO_CHANNEL_POSITION_LFE,
       },
+  {                             /* Not defined by spec, GStreamer default */
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_LFE,
+        GST_AUDIO_CHANNEL_POSITION_REAR_CENTER,
+      },
+  {                             /* Not defined by spec, GStreamer default */
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_LFE,
+        GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT,
+      },
 };
+
 static GstCaps *
 gst_vorbis_enc_generate_sink_caps (void)
 {
@@ -287,7 +303,7 @@ gst_vorbis_enc_generate_sink_caps (void)
           "endianness", G_TYPE_INT, G_BYTE_ORDER, "width", G_TYPE_INT, 32,
           NULL));
 
-  for (i = 3; i <= 6; i++) {
+  for (i = 3; i <= 8; i++) {
     GValue chanpos = { 0 };
     GValue pos = { 0 };
     GstStructure *structure;
@@ -313,7 +329,7 @@ gst_vorbis_enc_generate_sink_caps (void)
 
   gst_caps_append_structure (caps, gst_structure_new ("audio/x-raw-float",
           "rate", GST_TYPE_INT_RANGE, 1, 200000,
-          "channels", GST_TYPE_INT_RANGE, 7, 256,
+          "channels", GST_TYPE_INT_RANGE, 9, 256,
           "endianness", G_TYPE_INT, G_BYTE_ORDER, "width", G_TYPE_INT, 32,
           NULL));
 
@@ -468,6 +484,15 @@ gst_vorbis_enc_convert_sink (GstPad * pad, GstFormat src_format,
   return res;
 }
 
+static gint64
+gst_vorbis_enc_get_latency (GstVorbisEnc * vorbisenc)
+{
+  /* FIXME, this probably depends on the bitrate and other setting but for now
+   * we return this value, which was obtained by totally unscientific
+   * measurements */
+  return 58 * GST_MSECOND;
+}
+
 static const GstQueryType *
 gst_vorbis_enc_get_query_types (GstPad * pad)
 {
@@ -545,8 +570,28 @@ gst_vorbis_enc_src_query (GstPad * pad, GstQuery * query)
       gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
       break;
     }
+    case GST_QUERY_LATENCY:
+    {
+      gboolean live;
+      GstClockTime min_latency, max_latency;
+      gint64 latency;
+
+      if ((res = gst_pad_query (peerpad, query))) {
+        gst_query_parse_latency (query, &live, &min_latency, &max_latency);
+
+        latency = gst_vorbis_enc_get_latency (vorbisenc);
+
+        /* add our latency */
+        min_latency += latency;
+        if (max_latency != -1)
+          max_latency += latency;
+
+        gst_query_set_latency (query, live, min_latency, max_latency);
+      }
+      break;
+    }
     default:
-      res = gst_pad_query_default (pad, query);
+      res = gst_pad_query (peerpad, query);
       break;
   }
 
@@ -1117,10 +1162,29 @@ gst_vorbis_enc_chain (GstPad * pad, GstBuffer * buffer)
 
   if (vorbisenc->expected_ts != GST_CLOCK_TIME_NONE &&
       GST_BUFFER_TIMESTAMP (buffer) < vorbisenc->expected_ts) {
+    guint64 diff = vorbisenc->expected_ts - GST_BUFFER_TIMESTAMP (buffer);
+    guint64 diff_bytes;
+
     GST_WARNING_OBJECT (vorbisenc, "Buffer is older than previous "
-        "timestamp + duration, cannot handle. Dropping buffer.");
-    gst_buffer_unref (buffer);
-    return GST_FLOW_OK;
+        "timestamp + duration (%" GST_TIME_FORMAT "< %" GST_TIME_FORMAT
+        "), cannot handle. Clipping buffer.",
+        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
+        GST_TIME_ARGS (vorbisenc->expected_ts));
+
+    diff_bytes =
+        GST_CLOCK_TIME_TO_FRAMES (diff,
+        vorbisenc->frequency) * vorbisenc->channels * sizeof (gfloat);
+    if (diff_bytes >= GST_BUFFER_SIZE (buffer)) {
+      gst_buffer_unref (buffer);
+      return GST_FLOW_OK;
+    }
+    buffer = gst_buffer_make_metadata_writable (buffer);
+    GST_BUFFER_DATA (buffer) += diff_bytes;
+    GST_BUFFER_SIZE (buffer) -= diff_bytes;
+
+    GST_BUFFER_TIMESTAMP (buffer) += diff;
+    if (GST_BUFFER_DURATION_IS_VALID (buffer))
+      GST_BUFFER_DURATION (buffer) -= diff;
   }
 
   if (gst_vorbis_enc_buffer_check_discontinuous (vorbisenc, buffer) && !first) {
@@ -1172,6 +1236,8 @@ gst_vorbis_enc_chain (GstPad * pad, GstBuffer * buffer)
   /* tell the library how much we actually submitted */
   vorbis_analysis_wrote (&vorbisenc->vd, size);
 
+  GST_LOG_OBJECT (vorbisenc, "wrote %lu samples to vorbis", size);
+
   vorbisenc->samples_in += size;
 
   gst_buffer_unref (buffer);
