@@ -30,15 +30,25 @@ G_BEGIN_DECLS
 #define GST_TYPE_RING_BUFFER             (gst_ring_buffer_get_type())
 #define GST_RING_BUFFER(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_RING_BUFFER,GstRingBuffer))
 #define GST_RING_BUFFER_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_RING_BUFFER,GstRingBufferClass))
-#define GST_RING_BUFFER_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_RING_BUFFER, GstRingBufferClass))
+#define GST_RING_BUFFER_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_RING_BUFFER, GstRingBufferClass))
+#define GST_RING_BUFFER_CAST(obj)        ((GstRingBuffer *)obj)
 #define GST_IS_RING_BUFFER(obj)          (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_RING_BUFFER))
-#define GST_IS_RING_BUFFER_CLASS(obj)     (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_RING_BUFFER))
+#define GST_IS_RING_BUFFER_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_RING_BUFFER))
 
 typedef struct _GstRingBuffer GstRingBuffer;
 typedef struct _GstRingBufferClass GstRingBufferClass;
 typedef struct _GstRingBufferSpec GstRingBufferSpec;
 
-/* called to fill data with len bytes of samples */
+/**
+ * GstRingBufferCallback:
+ * @rbuf: a #GstRingBuffer
+ * @data: target to fill
+ * @len: amount to fill
+ * @user_data: user data
+ *
+ * This function is set with gst_ring_buffer_set_callback() and is
+ * called to fill the memory at @data with @len bytes of samples. 
+ */
 typedef void (*GstRingBufferCallback) (GstRingBuffer *rbuf, guint8* data, guint len, gpointer user_data);
 
 /**
@@ -52,7 +62,7 @@ typedef void (*GstRingBufferCallback) (GstRingBuffer *rbuf, guint8* data, guint
 typedef enum {
   GST_RING_BUFFER_STATE_STOPPED,
   GST_RING_BUFFER_STATE_PAUSED,
-  GST_RING_BUFFER_STATE_STARTED,
+  GST_RING_BUFFER_STATE_STARTED
 } GstRingBufferState;
 
 /**
@@ -68,7 +78,7 @@ typedef enum {
   GST_SEGSTATE_INVALID,
   GST_SEGSTATE_EMPTY,
   GST_SEGSTATE_FILLED,
-  GST_SEGSTATE_PARTIAL,
+  GST_SEGSTATE_PARTIAL
 } GstRingBufferSegState;
 
 /**
@@ -80,6 +90,10 @@ typedef enum {
  * @GST_BUFTYPE_IMA_ADPCM: samples in ima adpcm
  * @GST_BUFTYPE_MPEG: samples in mpeg audio format
  * @GST_BUFTYPE_GSM: samples in gsm format
+ * @GST_BUFTYPE_IEC958: samples in IEC958 frames (e.g. AC3)
+ * @GST_BUFTYPE_AC3: samples in AC3 format
+ * @GST_BUFTYPE_EAC3: samples in EAC3 format
+ * @GST_BUFTYPE_DTS: samples in DTS format
  *
  * The format of the samples in the ringbuffer.
  */
@@ -92,6 +106,10 @@ typedef enum
   GST_BUFTYPE_IMA_ADPCM,
   GST_BUFTYPE_MPEG,
   GST_BUFTYPE_GSM,
+  GST_BUFTYPE_IEC958,
+  GST_BUFTYPE_AC3,
+  GST_BUFTYPE_EAC3,
+  GST_BUFTYPE_DTS
 } GstBufferFormatType;
 
 typedef enum
@@ -140,9 +158,10 @@ typedef enum
   GST_IMA_ADPCM,
   GST_MPEG,
   GST_GSM,
-
-  /* fill me */
-
+  GST_IEC958,
+  GST_AC3,
+  GST_EAC3,
+  GST_DTS
 } GstBufferFormat;
 
 /**
@@ -156,12 +175,14 @@ typedef enum
  * @depth: th depth of the samples
  * @rate: the samplerate
  * @channels: the number of channels
- * @latency_time: the latency in time units
- * @buffer_time: the total buffer size in time units
+ * @latency_time: the latency in microseconds
+ * @buffer_time: the total buffer size in microseconds
  * @segsize: the size of one segment in bytes
  * @segtotal: the total number of segments
  * @bytes_per_sample: number of bytes in one sample
  * @silence_sample: bytes representing one sample of silence
+ * @seglatency: number of segments queued in the lower level device,
+ *  defaults to segtotal
  *
  * The structure containing the format specification of the ringbuffer.
  */
@@ -181,17 +202,30 @@ struct _GstRingBufferSpec
   gint      rate;
   gint      channels;
   
-  GstClockTime latency_time;    /* the required/actual latency time */
-  GstClockTime buffer_time;     /* the required/actual time of the buffer */
-  gint     segsize;             /* size of one buffer segement */
-  gint     segtotal;            /* total number of segments */
-
+  guint64  latency_time;        /* the required/actual latency time, this is the
+				 * actual the size of one segment and the
+				 * minimum possible latency we can achieve. */
+  guint64  buffer_time;         /* the required/actual time of the buffer, this is
+				 * the total size of the buffer and maximum
+				 * latency we can compensate for. */
+  gint     segsize;             /* size of one buffer segment in bytes, this value
+				 * should be chosen to match latency_time as
+				 * well as possible. */
+  gint     segtotal;            /* total number of segments, this value is the
+				 * number of segments of @segsize and should be
+				 * chosen so that it matches buffer_time as
+				 * close as possible. */
   /* out */
   gint     bytes_per_sample;    /* number of bytes of one sample */
   guint8   silence_sample[32];  /* bytes representing silence */
 
+  /* ABI added 0.10.20 */
+  gint     seglatency;          /* number of segments queued in the lower
+				 * level device, defaults to segtotal. */
+
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING];
+  /* gpointer _gst_reserved[GST_PADDING]; */
+  guint8 _gst_reserved[(sizeof (gpointer) * GST_PADDING) - sizeof (gint)];
 };
 
 #define GST_RING_BUFFER_GET_COND(buf) (((GstRingBuffer *)buf)->cond)
@@ -243,6 +277,8 @@ struct _GstRingBuffer {
   union {
     struct {
       gboolean           flushing;
+      /* ATOMIC */
+      gint               may_start;
     } ABI;
     /* adding + 0 to mark ABI change to be undone later */
     gpointer _gst_reserved[GST_PADDING + 0];
@@ -251,6 +287,7 @@ struct _GstRingBuffer {
 
 /**
  * GstRingBufferClass:
+ * @parent_class: parent class
  * @open_device:  open the device, don't set any params or allocate anything
  * @acquire: allocate the resources for the ringbuffer using the given spec
  * @release: free resources of the ringbuffer
@@ -325,16 +362,25 @@ void            gst_ring_buffer_clear_all       (GstRingBuffer *buf);
 /* commit samples */
 guint           gst_ring_buffer_commit          (GstRingBuffer *buf, guint64 sample, 
                                                  guchar *data, guint len);
+guint           gst_ring_buffer_commit_full     (GstRingBuffer * buf, guint64 *sample,
+		                                 guchar * data, gint in_samples, 
+						 gint out_samples, gint * accum);
+
 /* read samples */
 guint           gst_ring_buffer_read            (GstRingBuffer *buf, guint64 sample, 
                                                  guchar *data, guint len);
 
 /* mostly protected */
+/* not yet implemented
 gboolean        gst_ring_buffer_prepare_write   (GstRingBuffer *buf, gint *segment, guint8 **writeptr, gint *len);
+*/
 gboolean        gst_ring_buffer_prepare_read    (GstRingBuffer *buf, gint *segment, guint8 **readptr, gint *len);
 void            gst_ring_buffer_clear           (GstRingBuffer *buf, gint segment);
 void            gst_ring_buffer_advance         (GstRingBuffer *buf, guint advance);
 
+void            gst_ring_buffer_may_start       (GstRingBuffer *buf, gboolean allowed);
+
+
 G_END_DECLS
 
 #endif /* __GST_RING_BUFFER_H__ */
