@@ -25,6 +25,7 @@
 
 #include <string.h>
 #include <gst/gstutils.h>
+#include <gst/tag/tag.h>
 
 #include "riff-read.h"
 
@@ -89,7 +90,7 @@ skip_junk:
     goto too_small;
 
   *_chunk_data = buf;
-  *_offset += 8 + ((size + 1) & ~1);
+  *_offset += 8 + GST_ROUND_UP_2 (size);
 
   return GST_FLOW_OK;
 
@@ -165,7 +166,7 @@ gst_riff_parse_chunk (GstElement * element, GstBuffer * buf,
     *chunk_data = NULL;
 
   *_fourcc = fourcc;
-  *_offset += 8 + ((size + 1) & ~1);
+  *_offset += 8 + GST_ROUND_UP_2 (size);
 
   return TRUE;
 
@@ -577,78 +578,6 @@ too_small:
   }
 }
 
-static gchar *
-freeform_string_to_utf8 (const gchar * data, gint size, const gchar ** env_vars)
-{
-  const gchar *env = NULL;
-  gsize bytes_read;
-  gchar *utf8 = NULL;
-
-  g_return_val_if_fail (data != NULL, NULL);
-
-  if (size < 0)
-    size = strlen (data);
-
-  /* Should we try the charsets specified
-   * via environment variables FIRST ? */
-  if (g_utf8_validate (data, size, NULL))
-    return g_strndup (data, size);
-
-  while ((env == NULL || *env == '\0') && env_vars && *env_vars != NULL) {
-    env = g_getenv (*env_vars);
-    ++env_vars;
-  }
-
-  /* Try charsets specified via the environment */
-  if (env != NULL && *env != '\0') {
-    gchar **c, **csets;
-
-    csets = g_strsplit (env, G_SEARCHPATH_SEPARATOR_S, -1);
-
-    for (c = csets; c && *c; ++c) {
-      if ((utf8 = g_convert (data, size, "UTF-8", *c, &bytes_read, NULL, NULL))) {
-        if (bytes_read == size) {
-          g_strfreev (csets);
-          goto beach;
-        }
-        g_free (utf8);
-        utf8 = NULL;
-      }
-    }
-
-    g_strfreev (csets);
-  }
-
-  /* Try current locale (if not UTF-8) */
-  if (!g_get_charset (&env)) {
-    if ((utf8 = g_locale_to_utf8 (data, size, &bytes_read, NULL, NULL))) {
-      if (bytes_read == size) {
-        goto beach;
-      }
-      g_free (utf8);
-      utf8 = NULL;
-    }
-  }
-
-  /* Try ISO-8859-1 */
-  utf8 = g_convert (data, size, "UTF-8", "ISO-8859-1", &bytes_read, NULL, NULL);
-  if (utf8 != NULL && bytes_read == size) {
-    goto beach;
-  }
-
-  g_free (utf8);
-  return NULL;
-
-beach:
-
-  g_strchomp (utf8);
-  if (utf8 && utf8[0] != '\0')
-    return utf8;
-
-  g_free (utf8);
-  return NULL;
-}
-
 /**
  * gst_riff_parse_info:
  * @element: caller element (used for debugging/error).
@@ -727,7 +656,7 @@ gst_riff_parse_info (GstElement * element,
         type = GST_TAG_GENRE;
         break;
       case GST_RIFF_INFO_IKEY:
-        type = NULL; /*"Keywords"; */ ;
+        type = GST_TAG_KEYWORDS;
         break;
       case GST_RIFF_INFO_ILGT:
         type = NULL;            /*"Lightness"; */
@@ -776,7 +705,7 @@ gst_riff_parse_info (GstElement * element,
       };
       gchar *val;
 
-      val = freeform_string_to_utf8 ((gchar *) data, tsize, env_vars);
+      val = gst_tag_freeform_string_to_utf8 ((gchar *) data, tsize, env_vars);
 
       if (val) {
         gst_tag_list_add (taglist, GST_TAG_MERGE_APPEND, type, val, NULL);
