@@ -23,22 +23,17 @@
 /**
  * SECTION:element-adder
  *
- * <refsect2>
- * <para>
- * The Adder allows to mix several streams into one by adding the data.
+ * The adder allows to mix several streams into one by adding the data.
  * Mixed data is clamped to the min/max values of the data format.
- * </para>
+ *
+ * The adder currently mixes all data received on the sinkpads as soon as
+ * possible without trying to synchronize the streams.
+ *
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch audiotestsrc freq=100 ! adder name=mix ! audioconvert ! alsasink audiotestsrc freq=500 ! mix.
- * </programlisting>
- * This pipeline produces two sine waves mixed together.
- * </para>
- * <para>
- * The Adder currently mixes all data received on the sinkpads as soon as possible
- * without trying to synchronize the streams.
- * </para>
+ * ]| This pipeline produces two sine waves mixed together.
  * </refsect2>
  *
  * Last reviewed on 2006-05-09 (0.10.7)
@@ -115,17 +110,12 @@ GST_STATIC_PAD_TEMPLATE ("sink%d",
     );
 
 static void gst_adder_class_init (GstAdderClass * klass);
-
 static void gst_adder_init (GstAdder * adder);
-
 static void gst_adder_finalize (GObject * object);
 
 static gboolean gst_adder_setcaps (GstPad * pad, GstCaps * caps);
-
 static gboolean gst_adder_query (GstPad * pad, GstQuery * query);
-
 static gboolean gst_adder_src_event (GstPad * pad, GstEvent * event);
-
 static gboolean gst_adder_sink_event (GstPad * pad, GstEvent * event);
 
 static GstPad *gst_adder_request_new_pad (GstElement * element,
@@ -193,7 +183,6 @@ static GstCaps *
 gst_adder_sink_getcaps (GstPad * pad)
 {
   GstAdder *adder;
-
   GstCaps *result, *peercaps, *sinkcaps;
 
   adder = GST_ADDER (GST_PAD_PARENT (pad));
@@ -228,11 +217,8 @@ static gboolean
 gst_adder_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstAdder *adder;
-
   GList *pads;
-
   GstStructure *structure;
-
   const char *media_type;
 
   adder = GST_ADDER (GST_PAD_PARENT (pad));
@@ -342,13 +328,9 @@ static gboolean
 gst_adder_query_duration (GstAdder * adder, GstQuery * query)
 {
   gint64 max;
-
   gboolean res;
-
   GstFormat format;
-
   GstIterator *it;
-
   gboolean done;
 
   /* parse format */
@@ -418,13 +400,9 @@ static gboolean
 gst_adder_query_latency (GstAdder * adder, GstQuery * query)
 {
   GstClockTime min, max;
-
   gboolean live;
-
   gboolean res;
-
   GstIterator *it;
-
   gboolean done;
 
   res = TRUE;
@@ -449,11 +427,8 @@ gst_adder_query_latency (GstAdder * adder, GstQuery * query)
       case GST_ITERATOR_OK:
       {
         GstPad *pad = GST_PAD_CAST (item);
-
         GstQuery *peerquery;
-
         GstClockTime min_cur, max_cur;
-
         gboolean live_cur;
 
         peerquery = gst_query_new_latency ();
@@ -510,7 +485,6 @@ static gboolean
 gst_adder_query (GstPad * pad, GstQuery * query)
 {
   GstAdder *adder = GST_ADDER (gst_pad_get_parent (pad));
-
   gboolean res = FALSE;
 
   switch (GST_QUERY_TYPE (query)) {
@@ -579,7 +553,6 @@ static gboolean
 forward_event (GstAdder * adder, GstEvent * event)
 {
   gboolean ret;
-
   GstIterator *it;
   GValue vret = { 0 };
 
@@ -605,7 +578,6 @@ static gboolean
 gst_adder_src_event (GstPad * pad, GstEvent * event)
 {
   GstAdder *adder;
-
   gboolean result;
 
   adder = GST_ADDER (gst_pad_get_parent (pad));
@@ -618,9 +590,7 @@ gst_adder_src_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_SEEK:
     {
       GstSeekFlags flags;
-
       GstSeekType curtype;
-
       gint64 cur;
 
       /* parse the seek parameters */
@@ -636,7 +606,6 @@ gst_adder_src_event (GstPad * pad, GstEvent * event)
          * when all pads received a FLUSH_STOP. */
         gst_pad_push_event (adder->srcpad, gst_event_new_flush_start ());
       }
-
       /* now wait for the collected to be finished and mark a new
        * segment */
       GST_OBJECT_LOCK (adder->collect);
@@ -668,7 +637,6 @@ static gboolean
 gst_adder_sink_event (GstPad * pad, GstEvent * event)
 {
   GstAdder *adder;
-
   gboolean ret;
 
   adder = GST_ADDER (gst_pad_get_parent (pad));
@@ -700,15 +668,10 @@ gst_adder_sink_event (GstPad * pad, GstEvent * event)
 static void
 gst_adder_class_init (GstAdderClass * klass)
 {
-  GObjectClass *gobject_class;
+  GObjectClass *gobject_class = (GObjectClass *) klass;
+  GstElementClass *gstelement_class = (GstElementClass *) klass;
 
-  GstElementClass *gstelement_class;
-
-  gobject_class = (GObjectClass *) klass;
-
-  gobject_class->finalize = gst_adder_finalize;
-
-  gstelement_class = (GstElementClass *) klass;
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_adder_finalize);
 
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&gst_adder_src_template));
@@ -720,9 +683,10 @@ gst_adder_class_init (GstAdderClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gstelement_class->request_new_pad = gst_adder_request_new_pad;
-  gstelement_class->release_pad = gst_adder_release_pad;
-  gstelement_class->change_state = gst_adder_change_state;
+  gstelement_class->request_new_pad =
+      GST_DEBUG_FUNCPTR (gst_adder_request_new_pad);
+  gstelement_class->release_pad = GST_DEBUG_FUNCPTR (gst_adder_release_pad);
+  gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_adder_change_state);
 }
 
 static void
@@ -733,6 +697,7 @@ gst_adder_init (GstAdder * adder)
   template = gst_static_pad_template_get (&gst_adder_src_template);
   adder->srcpad = gst_pad_new_from_template (template, "src");
   gst_object_unref (template);
+
   gst_pad_set_getcaps_function (adder->srcpad,
       GST_DEBUG_FUNCPTR (gst_pad_proxy_getcaps));
   gst_pad_set_setcaps_function (adder->srcpad,
@@ -769,11 +734,8 @@ gst_adder_request_new_pad (GstElement * element, GstPadTemplate * templ,
     const gchar * unused)
 {
   gchar *name;
-
   GstAdder *adder;
-
   GstPad *newpad;
-
   gint padcount;
 
   if (templ->direction != GST_PAD_SINK)
@@ -848,17 +810,11 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
    * - push out the output buffer
    */
   GstAdder *adder;
-
   guint size;
-
   GSList *collected;
-
   GstBuffer *outbuf;
-
   GstFlowReturn ret;
-
   gpointer outbytes;
-
   gboolean empty = TRUE;
 
   adder = GST_ADDER (user_data);
@@ -881,11 +837,8 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
 
   for (collected = pads->data; collected; collected = g_slist_next (collected)) {
     GstCollectData *data;
-
     guint8 *bytes;
-
     guint len;
-
     GstBuffer *inbuf;
 
     data = (GstCollectData *) collected->data;
@@ -1020,7 +973,6 @@ static GstStateChangeReturn
 gst_adder_change_state (GstElement * element, GstStateChange transition)
 {
   GstAdder *adder;
-
   GstStateChangeReturn ret;
 
   adder = GST_ADDER (element);
