@@ -22,17 +22,15 @@
 /**
  * SECTION:element-audioresample
  *
- * <refsect2>
  * Audioresample resamples raw audio buffers to different sample rates using
  * a configurable windowing function to enhance quality.
+ *
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! audioresample ! audio/x-raw-int, rate=8000 ! alsasink
- * </programlisting>
- * Decode an Ogg/Vorbis downsample to 8Khz and play sound through alsa. 
+ * ]| Decode an Ogg/Vorbis downsample to 8Khz and play sound through alsa. 
  * To create the Ogg/Vorbis file refer to the documentation of vorbisenc.
- * </para>
  * </refsect2>
  *
  * Last reviewed on 2006-03-02 (0.10.4)
@@ -50,7 +48,7 @@
 #include <gst/audio/audio.h>
 #include <gst/base/gstbasetransform.h>
 
-GST_DEBUG_CATEGORY (audioresample_debug);
+GST_DEBUG_CATEGORY_STATIC (audioresample_debug);
 #define GST_CAT_DEFAULT audioresample_debug
 
 /* elementfactory information */
@@ -160,9 +158,9 @@ gst_audioresample_class_init (GstAudioresampleClass * klass)
   gobject_class->get_property = gst_audioresample_get_property;
 
   g_object_class_install_property (gobject_class, PROP_FILTERLEN,
-      g_param_spec_int ("filter_length", "filter_length", "filter_length",
-          0, G_MAXINT, DEFAULT_FILTERLEN,
-          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+      g_param_spec_int ("filter-length", "filter length",
+          "Length of the resample filter", 0, G_MAXINT, DEFAULT_FILTERLEN,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 
   GST_BASE_TRANSFORM_CLASS (klass)->start =
       GST_DEBUG_FUNCPTR (audioresample_start);
