@@ -128,27 +128,29 @@ gst_stream_info_class_init (GstStreamInfoClass * klass)
   g_object_class_install_property (gobject_klass, ARG_PAD,
       g_param_spec_object ("object", "object",
           "Source Pad or object of the stream", GST_TYPE_OBJECT,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_klass, ARG_TYPE,
       g_param_spec_enum ("type", "Type", "Type of the stream",
-          GST_TYPE_STREAM_TYPE, GST_STREAM_TYPE_UNKNOWN, G_PARAM_READABLE));
+          GST_TYPE_STREAM_TYPE, GST_STREAM_TYPE_UNKNOWN,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_klass, ARG_DECODER,
       g_param_spec_string ("decoder", "Decoder",
-          "The decoder used to decode the stream", NULL, G_PARAM_READABLE));
+          "The decoder used to decode the stream", NULL,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_klass, ARG_MUTE,
       g_param_spec_boolean ("mute", "Mute", "Mute or unmute this stream", FALSE,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_klass, ARG_CAPS,
       g_param_spec_boxed ("caps", "Capabilities",
           "Capabilities (or type) of this stream", GST_TYPE_CAPS,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_klass, ARG_LANG_CODE,
       g_param_spec_string ("language-code", "Language code",
-          "Language code for this stream, conforming to ISO-639-1",
-          NULL, G_PARAM_READABLE));
+          "Language code for this stream, conforming to ISO-639-1", NULL,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_klass, ARG_CODEC,
-      g_param_spec_string ("codec", "Codec",
-          "Codec used to encode the stream", NULL, G_PARAM_READABLE));
+      g_param_spec_string ("codec", "Codec", "Codec used to encode the stream",
+          NULL, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   gst_stream_info_signals[SIGNAL_MUTED] =
       g_signal_new ("muted", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
@@ -184,18 +186,28 @@ cb_probe (GstPad * pad, GstEvent * e, gpointer user_data)
 
     gst_event_parse_tag (e, &list);
 
-    if (gst_tag_list_get_string (list, GST_TAG_VIDEO_CODEC, &codec)) {
+    if (info->type != GST_STREAM_TYPE_AUDIO &&
+        gst_tag_list_get_string (list, GST_TAG_VIDEO_CODEC, &codec)) {
       g_free (info->codec);
       info->codec = codec;
+      GST_LOG_OBJECT (pad, "codec = %s (video)", codec);
       g_object_notify (G_OBJECT (info), "codec");
-    } else if (gst_tag_list_get_string (list, GST_TAG_AUDIO_CODEC, &codec)) {
+    } else if (info->type != GST_STREAM_TYPE_VIDEO &&
+        gst_tag_list_get_string (list, GST_TAG_AUDIO_CODEC, &codec)) {
       g_free (info->codec);
       info->codec = codec;
+      GST_LOG_OBJECT (pad, "codec = %s (audio)", codec);
+      g_object_notify (G_OBJECT (info), "codec");
+    } else if (gst_tag_list_get_string (list, GST_TAG_CODEC, &codec)) {
+      g_free (info->codec);
+      info->codec = codec;
+      GST_LOG_OBJECT (pad, "codec = %s (generic)", codec);
       g_object_notify (G_OBJECT (info), "codec");
     }
     if (gst_tag_list_get_string (list, GST_TAG_LANGUAGE_CODE, &lang)) {
       g_free (info->langcode);
       info->langcode = lang;
+      GST_LOG_OBJECT (pad, "language-code = %s", lang);
       g_object_notify (G_OBJECT (info), "language-code");
     }
   }
@@ -241,7 +253,7 @@ gst_stream_info_dispose (GObject * object)
         GST_PAD_CAST (stream_info->object));
     if (parent != NULL) {
       g_signal_handlers_disconnect_by_func (parent,
-          G_CALLBACK (stream_info_change_state), stream_info);
+          (gpointer) stream_info_change_state, stream_info);
       gst_object_unref (parent);
     }
 
@@ -292,25 +304,33 @@ gst_stream_info_set_mute (GstStreamInfo * stream_info, gboolean mute)
   }
 
   if (mute != stream_info->mute) {
-    GstElement *element;
+    /* nothing really happens here. it looks like gstplaybasebin installs a
+     * buffer probe hat drops buffers when muting. but the this removes it self
+     * after first call.
+     */
 
     stream_info->mute = mute;
-    // gst_pad_set_active ((GstPad *) GST_PAD_CAST (stream_info->object), !mute);
-
-    element = gst_pad_get_parent_element ((GstPad *)
-        GST_PAD_CAST (stream_info->object));
-    if (element) {
 #if 0
-      if (mute) {
-        g_signal_connect (element, "state-changed",
-            G_CALLBACK (stream_info_change_state), stream_info);
-      } else {
-        g_signal_handlers_disconnect_by_func (element,
-            G_CALLBACK (stream_info_change_state), stream_info);
-      }
+    gst_pad_set_active ((GstPad *) GST_PAD_CAST (stream_info->object), !mute);
 #endif
-      gst_object_unref (element);
+#if 0
+    {
+      GstElement *element;
+
+      element = gst_pad_get_parent_element ((GstPad *)
+          GST_PAD_CAST (stream_info->object));
+      if (element) {
+        if (mute) {
+          g_signal_connect (element, "state-changed",
+              G_CALLBACK (stream_info_change_state), stream_info);
+        } else {
+          g_signal_handlers_disconnect_by_func (element,
+              G_CALLBACK (stream_info_change_state), stream_info);
+        }
+        gst_object_unref (element);
+      }
     }
+#endif
   }
   return TRUE;
 }
